@@ -28,7 +28,7 @@ ALLOWED_PDF_EXTENSIONS = {'.csv', '.doc', '.docx', '.odp', '.ods', '.odt',
 class DownloadableMixin:
 
     def download(self, to_path=None, name=None, chunk_size='auto',
-                 convert_to_pdf=False):
+                 convert_to_pdf=False, output=None):
         """ Downloads this file to the local drive. Can download the
         file in chunks with multiple requests to the server.
 
@@ -41,26 +41,29 @@ class DownloadableMixin:
          however only 1 request)
         :param bool convert_to_pdf: will try to download the converted pdf
          if file extension in ALLOWED_PDF_EXTENSIONS
+        :param RawIOBase output: (optional) an opened io object to write to.
+         if set, the to_path and name will be ignored
         :return: Success / Failure
         :rtype: bool
         """
         # TODO: Add download with more than one request (chunk_requests) with
         #  header 'Range'. For example: 'Range': 'bytes=0-1024'
 
-        if to_path is None:
-            to_path = Path()
-        else:
-            if not isinstance(to_path, Path):
-                to_path = Path(to_path)
+        if not output:
+            if to_path is None:
+                to_path = Path()
+            else:
+                if not isinstance(to_path, Path):
+                    to_path = Path(to_path)
 
-        if not to_path.exists():
-            raise FileNotFoundError('{} does not exist'.format(to_path))
+            if not to_path.exists():
+                raise FileNotFoundError('{} does not exist'.format(to_path))
 
-        if name and not Path(name).suffix and self.name:
-            name = name + Path(self.name).suffix
+            if name and not Path(name).suffix and self.name:
+                name = name + Path(self.name).suffix
 
-        name = name or self.name
-        to_path = to_path / name
+            name = name or self.name
+            to_path = to_path / name
 
         url = self.build_url(
             self._endpoints.get('download').format(id=self.object_id))
@@ -89,14 +92,22 @@ class DownloadableMixin:
                     log.debug('Downloading driveitem Request failed: {}'.format(
                         response.reason))
                     return False
-                with to_path.open(mode='wb') as output:
+
+                def write_output(out):
                     if stream:
                         for chunk in response.iter_content(
                                 chunk_size=chunk_size):
                             if chunk:
-                                output.write(chunk)
+                                out.write(chunk)
                     else:
-                        output.write(response.content)
+                        out.write(response.content)
+
+                if output:
+                    write_output(output)
+                else:
+                    with to_path.open(mode='wb') as output:
+                        write_output(output)
+
         except Exception as e:
             log.error(
                 'Error downloading driveitem {}. Error: {}'.format(self.name,
@@ -463,6 +474,7 @@ class DriveItem(ApiComponent):
         parent_reference = cloud_data.get(self._cc('parentReference'), {})
         self.parent_id = parent_reference.get('id', None)
         self.drive_id = parent_reference.get(self._cc('driveId'), None)
+        self.parent_path = parent_reference.get(self._cc("path"), None)
 
         remote_item = cloud_data.get(self._cc('remoteItem'), None)
         if remote_item is not None:
@@ -1008,7 +1020,7 @@ class Folder(DriveItem):
             'name', None)
 
     def get_items(self, limit=None, *, query=None, order_by=None, batch=None):
-        """ Returns all the items inside this folder
+        """ Returns generator all the items inside this folder
 
         :param int limit: max no. of folders to get. Over 999 uses batch.
         :param query: applies a OData filter to the request
@@ -1017,8 +1029,8 @@ class Folder(DriveItem):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
 
         url = self.build_url(
@@ -1070,8 +1082,8 @@ class Folder(DriveItem):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: folder items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
 
         if query:
@@ -1112,7 +1124,6 @@ class Folder(DriveItem):
     def download_contents(self, to_folder=None):
         """ This will download each file and folder sequentially.
         Caution when downloading big folder structures
-
         :param drive.Folder to_folder: folder where to store the contents
         """
         if to_folder is None:
@@ -1121,13 +1132,22 @@ class Folder(DriveItem):
             except Exception as e:
                 log.error('Could not create folder with name: {}. Error: {}'.format(self.name, e))
                 to_folder = Path()  # fallback to the same folder
-
-        if not to_folder.exists():
-            to_folder.mkdir()
-
+        else:
+            to_folder = Path() / to_folder
+            if not to_folder.exists():
+                to_folder.mkdir()
+        if not isinstance(to_folder,str):
+            if not to_folder.exists():
+                to_folder.mkdir()
+        else:
+            to_folder = Path() / self.name
+        
         for item in self.get_items(query=self.new_query().select('id', 'size', 'folder', 'name')):
             if item.is_folder and item.child_count > 0:
                 item.download_contents(to_folder=to_folder / item.name)
+            elif item.is_folder and item.child_count == 0:
+                if not to_folder.exists():
+                    to_folder.mkdir()
             else:
                 item.download(to_folder)
 
@@ -1150,8 +1170,8 @@ class Folder(DriveItem):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder matching search
+        :rtype: generator of DriveItem or Pagination
         """
         if not isinstance(search_text, str) or not search_text:
             raise ValueError('Provide a valid search_text')
@@ -1198,30 +1218,39 @@ class Folder(DriveItem):
             return items
 
     def upload_file(self, item, item_name=None, chunk_size=DEFAULT_UPLOAD_CHUNK_SIZE,
-                    upload_in_chunks=False):
+                    upload_in_chunks=False, stream=None, stream_size=None,
+                    conflict_handling=None):
         """ Uploads a file
 
         :param item: path to the item you want to upload
         :type item: str or Path
-        :param item: name of the item on the server. None to use original name
-        :type item: str or Path
-        :param chunk_size: Only applies if file is bigger than 4MB.
+        :param item_name: name of the item on the server. None to use original name
+        :type item_name: str or Path
+        :param chunk_size: Only applies if file is bigger than 4MB or upload_in_chunks is True.
          Chunk size for uploads. Must be a multiple of 327.680 bytes
         :param upload_in_chunks: force the method to upload the file in chunks
+        :param io.BufferedIOBase stream: (optional) an opened io object to read into.
+         if set, the to_path and name will be ignored
+        :param int stream_size: size of stream, required if using stream
+        :param conflict_handling: How to handle conflicts.
+         NOTE: works for chunk upload only (>4MB or upload_in_chunks is True)
+         None to use default (overwrite). Options: fail | replace | rename
+        :type conflict_handling: str
         :return: uploaded file
         :rtype: DriveItem
         """
 
-        if item is None:
-            raise ValueError('Item must be a valid path to file')
-        item = Path(item) if not isinstance(item, Path) else item
+        if not stream:
+            if item is None:
+                raise ValueError('Item must be a valid path to file')
+            item = Path(item) if not isinstance(item, Path) else item
 
-        if not item.exists():
-            raise ValueError('Item must exist')
-        if not item.is_file():
-            raise ValueError('Item must be a file')
+            if not item.exists():
+                raise ValueError('Item must exist')
+            if not item.is_file():
+                raise ValueError('Item must be a file')
 
-        file_size = item.stat().st_size
+        file_size = (stream_size if stream_size is not None else item.stat().st_size)
 
         if not upload_in_chunks and file_size <= UPLOAD_SIZE_LIMIT_SIMPLE:
             # Simple Upload
@@ -1231,8 +1260,11 @@ class Folder(DriveItem):
             # headers = {'Content-type': 'text/plain'}
             headers = {'Content-type': 'application/octet-stream'}
             # headers = None
-            with item.open(mode='rb') as file:
-                data = file.read()
+            if stream:
+                data = stream.read()
+            else:
+                with item.open(mode='rb') as file:
+                    data = file.read()
 
             response = self.con.put(url, headers=headers, data=data)
             if not response:
@@ -1246,9 +1278,14 @@ class Folder(DriveItem):
             # Resumable Upload
             url = self.build_url(
                 self._endpoints.get('create_upload_session').format(
-                    id=self.object_id, filename=quote(item.name)))
+                    id=self.object_id, filename=quote(item.name if item_name is None else item_name)))
 
-            response = self.con.post(url)
+            # If not None, add conflict handling to request
+            file_data = {}
+            if conflict_handling:
+                file_data["item"] = {"@microsoft.graph.conflictBehavior": conflict_handling }
+
+            response = self.con.post(url, data=file_data)
             if not response:
                 return None
 
@@ -1264,8 +1301,8 @@ class Folder(DriveItem):
                           'upload_url for file {}'.format(item.name))
                 return None
 
-            current_bytes = 0
-            with item.open(mode='rb') as file:
+            def write_stream(file):
+                current_bytes = 0
                 while True:
                     data = file.read(chunk_size)
                     if not data:
@@ -1295,6 +1332,11 @@ class Folder(DriveItem):
                         data = response.json()
                         return self._classifier(data)(parent=self, **{
                             self._cloud_data_key: data})
+            if stream:
+                return write_stream(stream)
+            else:
+                with item.open(mode='rb') as file:
+                    return write_stream(file)
 
 
 class Drive(ApiComponent):
@@ -1460,8 +1502,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
 
         if self.object_id:
@@ -1485,8 +1527,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: folder items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
 
         if query:
@@ -1506,8 +1548,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
         if self.object_id:
             # reference the current drive_id
@@ -1531,8 +1573,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder
+        :rtype: generator of DriveItem or Pagination
         """
 
         if self.object_id:
@@ -1687,8 +1729,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
-        :return: list of items in this folder
-        :rtype: list[DriveItem] or Pagination
+        :return: items in this folder matching search
+        :rtype: generator of DriveItem or Pagination
         """
         if not isinstance(search_text, str) or not search_text:
             raise ValueError('Provide a valid search_text')
